#include <VRS51L3074_SDCC.h>


#define RXBUF1SZ 16

__xdata unsigned char ser1rxbuf[RXBUF1SZ];
static __xdata unsigned char *txptr,*rxptr;
unsigned char ser1txbytes,ser1rxbytes;

/*
   Rx is always enabled, flush the buffer with ser1flush(), check
   available chars watching ser1rxbytes, peek last received char
   calling ser1peek(). Data is at ser1rxbuf in xram
   
   Enable Tx by calling ser1send with a pointer to data in xram
   and the number of bytes to send. Check if finished by watching
   ser1txbytes, should be 0 when finished (or check UART1INT bit 4, BTW).
*/

void ser1Interrupt(void) interrupt 6
{
register unsigned char st,dat;
  
    st=UART1INT;
    if(st&0x6){                // RXAVENF or RXOVF
      dat=UART1BUF;             // read it 
      if(ser1rxbytes<RXBUF1SZ){     // avoid buffer overflow
	*(rxptr++)=dat;             // put it in the buffer
        if(st&(1<<2))
	  rxptr--;                      // ignore on RxOV int (but read anyway)
        else
	  ser1rxbytes++;
      }
    }
    else {                           // TXEMPTYF
      if(ser1txbytes){
	UART1BUF=*(txptr++);           // if bytes, send them
	--ser1txbytes;
      }
      if(!ser1txbytes)
	UART1INT=0x62;           // if no more bytes, disable interrupt
    }
}

// Returns -1 if there is no char available
int ser1peek(void)
{
  if(ser1rxbytes)
    return(*(rxptr-1));
  else 
    return(-1);
}


void ser1flush(void)
{
register unsigned char aux;

  aux=UART1INT;
  aux&=0x70;                 // mask out non interrupt bits
  aux|=0x02;                  // keep reception enabled
  UART1INT=(aux&~0x60);       // disable RXAVENF + RXOVF interrupts
  rxptr=ser1rxbuf;
  ser1rxbytes=0;
  UART1INT=aux;            // enable RXAVENF + RXOVF interrupts + reception
}

void ser1send(__xdata unsigned char *ptr,unsigned char bytes)
{
  UART1INT=0x62;           // disable TXEMPTY interrupt
  txptr=ptr;
  ser1txbytes=bytes;
  UART1INT=0x72;           // enable TXEMPTY interrupt
}

void ser1init(unsigned long clockspeed)
{
register unsigned char aux;
register unsigned int aux2;
  
	PERIPHEN1 |= (1<<4);			// Enable UART1

	aux=(unsigned char)(clockspeed>>1)&0x0F; // Fine adjustment on baud rate
	UART1CFG = (aux<<4);			//Use internal clock
						//9th bit not used
						//only one stop bit

	UART1INT = 0x62;			//Enable RX AV + RXO V int + Enable Reception
	UART1EXT = 0x00;			//Not using UART1 Extensions

	aux2=((unsigned int)(clockspeed>>5)&0xFFFF)-1;   // BRG = (clock/speed)/32 -1
	UART1BRL = (unsigned char)aux2&0xFF;
	UART1BRH = (unsigned char)(aux2>>8)&0xFF;
	ser1flush();
	INTSRC1 &= ~(1<<6);                   // UART1 module interrupt
	INTEN1 |= (1<<6);                     // interrupt 6 enable
}

